##
## Copyright (c) 2001, 2002, Amnon BARAK (amnon@MOSIX.org).
## All rights reserved.
## This software is subject to the MOSIX SOFTWARE LICENSE AGREEMENT.
##
## THIS SOFTWARE IS PROVIDED IN ITS "AS IS" CONDITION, WITH NO WARRANTY
## WHATSOEVER. NO LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
## FROM THE USE OF THIS SOFTWARE WILL BE ACCEPTED.
##
##
## Author(s): Shlomo Matichin 
##

# this file includes the common routines and startup checks for the MOSIX
# automatic installation utility scripts.

package common;
require Exporter;

#to use this module, use the follwing three lines:
#use FindBin;
#use lib "$FindBin::Bin";
#use common;



our @ISA = qw(Exporter);
our @EXPORT = qw($suse $redhat $debian $distribution dead find_root_dev pipe_stderr_too $mosix_version $kernel_version $inst verify_kernel_version safe_pipe_stderr_too patch_distribution $source_dir $initd $rcd verify_initd_rcd trace_link);



use File::Basename;   
use FindBin;
$inst = $FindBin::Bin;
$source_dir = glob "$inst/..";

#the dead func
sub dead {
	print STDERR "\n" , @_ , "\n" ;
	exit 1 if $! == 0;
	exit $! ;
}



#find_root_dev
#this func find the root device
sub find_root_dev {
    my $answer = `df /`;
    $answer =~ /.*(\/dev\/\w*)/;
    return $1;
}

sub pipe_stderr_too {
	#first argument: command line to run
	#returns: like back ticks but with stderr too
	open( FILE , "$_[0] 2>&1 |" );
	@lines = <FILE>;
	close( FILE );
	$line = join '' , @lines ;
	return $line;
}

sub safe_pipe_stderr_too {
	$command = shift;
	if( not open( FILE , "$command 2>&1 |" ) ) {
		$name = (split /\ /,$command)[0];
		`which $name 2>&1`;
		$? and dead "Executing `$command` failed: the command `$name` was not found. `$name` is required for the automatic MOSIX installation utility\n";
		dead "Executing `$command` failed: unable to execute\n";
	}
	@lines = <FILE>;
	close( FILE );
	$line = join '' , @lines ;
	$? and dead "Executing `$command` failed with return code $?. ".
			"Output: \"$line\"\n";
	return $line
}

#check for root permissions
safe_pipe_stderr_too( "whoami" ) =~ /^root\n/ or dead "Only root may run this script\n";

#$mosix_version & $kernel_version
#read the DISTRIBUTION file for the versions
-e "$inst/DISTRIBUTION" or
	dead "MOSIX Distribution is not intact: Unable to find $inst/DISTRIBUTION";
$versions = safe_pipe_stderr_too "cat $inst/DISTRIBUTION" ;
$versions =~ /MOSIX_VERSION\s*\=\s*(.*)\n/ or
	dead "Error in $inst/DISTRIBUTION format!";
$mosix_version = $1;
$mosix_version =~ /^\d+\.\d+\.\d+/ or
	dead "Error in $inst/DISTRIBUTION format!";
$versions =~ /KERNEL_VERSION\s*\=\s*(.*)\n/ or
	dead "Error in $inst/DISTRIBUTION format!";
$kernel_version = $1;
$kernel_version =~ /^\d+\.\d+\.\d+$/ or
	dead "Error in $inst/DISTRIBUTION format!";

sub verify_kernel_version {
	$kernel_head = safe_pipe_stderr_too "head $_[0]/Makefile";
	($version , $patch , $sub ) = split /\./ , $kernel_version;
	$kernel_head =~ /VERSION\s\=\s$version/ and
	$kernel_head =~ /PATCHLEVEL\s\=\s$patch/ and
	$kernel_head =~ /SUBLEVEL\s\=\s$sub/ or
		dead "Kernel version does not match MOSIX distribution";
	return 0;
}

#get the distribution variables
$debian = $suse = $redhat = 0;
-e "/etc/redhat-release" and ($redhat = 1) and $distribution = "redhat";
-e "/etc/SuSE-release" and ($suse = 1) and $distribution = "suse";
-e "/etc/debian_version" and ($debian = 1) and $distribution = "debian";
defined $distribution or $distribution = "untested";

# init.d and rc.d location:
-d "/etc/init.d" and ($initd = "/etc/init.d") or
	-d "/etc/rc.d/init.d" and ($initd = "/etc/rc.d/init.d");
-d "/etc/rc.d" and ($rcd = "/etc/rc.d");

sub verify_initd_rcd {
	defined $initd or dead "Unable to find the init.d directory. Please specify the --initd parameter";
	defined $rcd or dead "Unable to find the rc.d directory. Please specify the --rcd parameter";
	$initd =~ s/\/$// ;
	$rcd =~ s/\/$// ;
	-d $initd or dead "Directory `$initd' does not exist";
	-d $rcd or dead "Directory `$rcd' does not exists";
}

sub patch_distribution {
	$tar_file = shift;
	$new_dir = shift;
	$files = safe_pipe_stderr_too "tar -t -f $tar_file";
	@filelist = split /\n/ , $files ;
	@filelist_patches = grep /^patch/ , @filelist;
	@filelist_patches_relevent = grep /$distribution/ , @filelist_patches;
        if (@filelist_patches_relevent) {
                foreach $fname(@filelist_patches_relevent) {
                        $bname = basename $fname;
                        $dname = dirname $fname;
                        chdir("$new_dir$dname") or dead "Unable to change dir into '$newdir$dname'";
                        system("patch -p0 --dry-run < $bname") and 
				dead "The '$new_dir$fname' patch failed";
                        print("patching $distribution files...\n");
                        system("patch -p0 < $bname  ");
                }
        }
	if(@filelist_patches) {
		foreach $fname(@filelist_patches) {
			unlink( "$new_dir/$fname" );
		}
	}
        return 0;
}

sub trace_link {
	my $name = shift;
	while( readlink $name ) {
		$name = readlink $name;
	}
	return $name
}

1;

